local function update(self, dt)
  self.x = self.x + self.vx * dt
  self.y = self.y + self.vy * dt
end

local function draw(self,debug)
  love.graphics.setColor(0,0,0,255)
  love.graphics.circle("fill", self.x, self.y, self.r)
  if debug then
    love.graphics.setColor(255,255,128,50)
    love.graphics.circle("line", self.x, self.y, self.r)
  end
end

local blackholes = {}

function blackholes:new(x,y,r,vx,vy)
  local blackhole = {}
  blackhole.draw = draw
  blackhole.update = update
  blackhole.x = x or 0
  blackhole.y = y or 0
  blackhole.vx = vx or 0
  blackhole.vy = vy or 0
  blackhole.r = r or 1
  blackhole.m = 2*r
  blackhole.kill = false
  table.insert(self.all, blackhole)
  return blackhole
end

function blackholes:init()
  self.all = {}
end

function blackholes:update(physics,dt)
  local marked = {}
  
  for i=#self.all,1,-1 do
    local blackhole = self.all[i]
    marked[i] = true

    for j=1,#self.all do
      if marked[j] ~= true then
        local other = self.all[j]
        local collision = physics:gravity(blackhole,other,dt,true)
        
        if collision then
          local pow = 6
          local dr = math.pow(math.pow(blackhole.r,pow) + math.pow(other.r,3),1/pow)
          print(dr)
          if blackhole.r > other.r then
            blackhole.r = blackhole.r + dr
            other.kill = true
          else
            other.r = other.r + dr
            blackhole.kill = true
          end
        end
      end
    end
    
    blackhole:update(dt) -- velocity only
    if blackhole.kill then
      table.remove(self.all, i)
    end
  end
end

function blackholes:draw(debug)
  for i=1, #self.all do
    local blackhole = self.all[i]
    blackhole:draw(debug)
  end
end

return blackholes