local camera = {}

function camera:init(s,x,y)
  self.scale = s or 1
  self.x = x or 0
  self.y = y or 0
  self.zoomSpeed = 2
  self.translateSpeed = 1000
  self.followed = nil
end

function camera:zoom(dir,howmuch,w,h)
  local coef = howmuch / (2 * self.scale)
  if dir == "forward" then
    self.scale = self.scale * (1 + howmuch)
    self.x = self.x + w * coef
    self.y = self.y + h * coef
  elseif dir == "backward" then
    self.scale = self.scale / (1 + howmuch)
    self.x = self.x - w * coef
    self.y = self.y - h * coef
  end
end

function camera:translate(dir,howmuch)
  if dir == "right" then
    self.x = self.x + howmuch
  end
  if dir == "left" then
    self.x = self.x - howmuch
  end
  if dir == "down" then
    self.y = self.y + howmuch
  end
  if dir == "up" then
    self.y = self.y - howmuch
  end
end

function camera:wheel(y,w,h)
  local dir = "none"
  if y > 0 then
    dir = "forward"
  elseif y < 0 then
    dir = "backward"
  end
  local ratio = self.zoomSpeed / 16
  self:zoom(dir,ratio,w,h)
end

function camera:update(w,h,dt)
  -- translate
  local speed = self.translateSpeed / self.scale * dt
  --  mouse
  local mousex = love.mouse.getX()
  local mousey = love.mouse.getY()
  if mousex == 0 then
    camera:translate("left",speed)
  elseif mousex == w-1 then
    camera:translate("right",speed)
  end
  if mousey == 0 then
    camera:translate("up",speed)
  elseif mousey == h-1 then
    camera:translate("down",speed)
  end
  --  keyboard
  if love.keyboard.isDown("right") then
    camera:translate("right",speed)
  end
  if love.keyboard.isDown("left") then
    camera:translate("left",speed)
  end
  if love.keyboard.isDown("down") then
    camera:translate("down",speed)
  end
  if love.keyboard.isDown("up") then
    camera:translate("up",speed)
  end
  -- zoom
  local ratio = self.zoomSpeed * dt
  if love.keyboard.isDown("lshift") then
    self:zoom("forward",ratio,w,h)
  end
  if love.keyboard.isDown("lctrl") then
    self:zoom("backward",ratio,w,h)
  end
  if self.followed ~= nil then
    camera.x = self.followed.x - w/2 / camera.scale
    camera.y = self.followed.y - h/2 / camera.scale
  end
end

function camera:follow(planets,blackholes,stars,mx,my)
  local x = camera.x + mx / camera.scale
  local y = camera.y + my / camera.scale  
  -- planets
  for i=1,#planets.all do
    local planet = planets.all[i]
    local dx = planet.x - x
    local dy = planet.y - y
    local d = math.sqrt(dx*dx + dy*dy)
    if d < planet.r then
      self.followed = planet
      return
    end
  end
  -- stars
  for i=1,#stars.all do
    local star = stars.all[i]
    local dx = star.x - x
    local dy = star.y - y
    local d = math.sqrt(dx*dx + dy*dy)
    if d < star.r then
      self.followed = star
      return
    end
  end
  -- holes
  for i=1,#blackholes.all do
    local blackhole = blackholes.all[i]
    local dx = blackhole.x - x
    local dy = blackhole.y - y
    local d = math.sqrt(dx*dx + dy*dy)
    if d < blackhole.r then
      self.followed = blackhole
      return
    end
  end
  
  self.followed = nil
end

return camera