local function split(s)
  result = {};
  local len = string.len(s)
  for i=1,len do
    result[i] = string.sub(s,i,i)
  end
  return result;
end

local displays = {}

function displays:init(what,w,h)
  self.all = {}
  love.graphics.setNewFont(20)
  self.hideAlpha = 1
  
  self:new("Gravity Playground",10,10,{128,200,255,128},true)
  
  self.planet = self:new("1 Planet",20,45,{165,214,219,128})
  self.planet.borders = true
  self.blackhole = self:new("2 BlackHole",20,80,{255,255,255,128})
  self.star = self:new("3 Star",20,115,{255,141,0,128})
  
  self:new("Escape | Quit",10,h-200,nil,false,true)
  self:new("R | New Game",10,h-165,nil,false,true)
  self:new("D | Debug Mode",10,h-130,nil,false,true)
  self:new("Scroll | (De)Zoom",10,h-95,nil,false,true)
  self:new("Right Click | (Un)Follow",10,h-60,nil,false,true)
  self:new("Left Click  | Spawn (drag for velocity)",10,h-25,nil,false,true)
  
  self:focus(what)
end

function displays:new(t,x,y,c,u,hide)
  local text = {}
  text.content = split(t) or ""
  text.x = x or 0
  text.y = y or 0
  text.c = c or {255,255,255,128}
  text.u = u or false
  text.boders = false
  text.hide = hide or false
  text.size = 20
  table.insert(self.all, text)
  return text
end

function displays:update(my,h)
  self.hideAlpha = h/my
end

function displays:focus(what)
  self.planet.borders = false
  self.blackhole.borders = false
  self.star.borders = false
  
  if what == "planet" then
    self.planet.borders = true
  elseif what == "blackhole" then
    self.blackhole.borders = true
  elseif what == "star" then
    self.star.borders = true
  end
end

function displays:draw(scale)
  love.graphics.push()
  love.graphics.scale(1/scale,1/scale)
  
  for i=1,#self.all do
    local text = self.all[i]
    
    if text.hide == true then
      love.graphics.setColor(text.c[1], text.c[2], text.c[3], text.c[4] / self.hideAlpha)
    else
      love.graphics.setColor(text.c)
    end
    
    local x = text.x
    local y = text.y
    local len = #text.content
    for i=1,len do
      local char = text.content[i]
      love.graphics.print(char,x,y)
      x = x + text.size
    end
    
    if text.borders == true then
      local x = text.x - text.size * 0.1
      local h = text.size * 1.2
      local y = text.y - text.size * 0.1
      local w = (len) * text.size
      love.graphics.rectangle("line",x,y,w,h)
    elseif text.u == true then
      local x1 = text.x
      local y1 = y + text.size * 1.2
      local x2 = len * text.size
      local y2 = y1
      love.graphics.line(x1,y1,x2,y2)
    end
  end
  love.graphics.pop()
end

return displays