local colors = require("colors")
local physics = require("physics")
local background = require("background")
local blackholes = require("blackholes")
local planets = require("planets")
local stars = require("stars")
local particles = require("particles")
local camera = require("camera")
local spawner = require("spawner")
local displays = require("displays")

local screenW
local screenH
local debug
local gScale

function initGame()
  background:init(screenW*2,screenH*2)
  physics:init(100)
  planets:init()
  blackholes:init()
  particles:init()
  stars:init()
  camera:init()
  displays:init("planet",screenW,screenH)
  spawner:init("planet")
end

function love.load()
  debug = false
  gScale = 1
  --love.graphics.setDefaultFilter("nearest")
  love.graphics.setPointSize(2)
  math.randomseed( os.time() )
  love.window.setMode(1080/gScale,768/gScale)
  if gScale == 1 then
    love.window.setFullscreen(true)
  end
  screenW = love.graphics.getWidth()
  screenH = love.graphics.getHeight()
  initGame()
end

function love.mousepressed(x,y,b)
  if b == 1 then
    spawner:pressed(camera,x,y)
  else
    camera:follow(planets,blackholes,stars,x,y)
  end
end

function love.mousereleased(x,y,b)
  if b == 1 then
    spawner:released(camera,colors,planets,blackholes,stars,x,y)
  end
end

function love.wheelmoved(x,y)
  camera:wheel(y,screenW,screenH)
end

function love.keypressed(k)
  if k == "escape" then
    love.event.quit()
  elseif k == "r" then
    initGame()
  elseif k == "d" then
    debug = not debug
  elseif k == "1" then
    spawner:change("planet")
    displays:focus("planet")
  elseif k == "2" then
    spawner:change("blackhole")
    displays:focus("blackhole")
  elseif k == "3" then
    spawner:change("star")
    displays:focus("star")
  end
end

function love.update(dt)
  background:update(camera, dt)
  particles:update(planets, blackholes, stars, physics, dt)
  planets:update(particles, physics, blackholes, stars, colors, dt)
  blackholes:update(physics,dt)
  stars:update(particles,physics,blackholes,dt)
  camera:update(screenW, screenH ,dt)
  spawner:update(dt)
  displays:update(love.mouse.getY(),screenH)
end

function love.draw()
  displays:draw(gScale,screenW,screenH)
  love.graphics.push()
  love.graphics.scale(camera.scale, camera.scale)
  love.graphics.translate(-camera.x, -camera.y)
  background:draw()
  planets:draw(colors, camera, screenW, screenH, debug)
  stars:draw(camera,screenW,screenH,debug)
  blackholes:draw(debug)
  particles:draw(colors,debug)
  spawner:draw(camera)
  love.graphics.pop()
  love.graphics.setColor(255,255,255)
end