local particles = {}

function particles:init()
  self.all = {}
end

function particles:new(x,y,vx,vy,c)
  local particle = {}
  local coef = (10000 - #self.all)/(10000 + #self.all) - 1/2
  if coef > 0 then
    particle.duration = math.random(100,200)*coef
    particle.timer = particle.duration
    particle.kill = false
    particle.x = x or 0
    particle.y = y or 0
    particle.vx = vx or 0
    particle.vy = vy or 0
    particle.r = 1
    particle.m = 50
    particle.c = c or {0,0,255}
    table.insert(self.all, particle)
    return particle
  end
end

function particles:update(planets,blackholes,stars,physics,dt)
  for i=#self.all,1,-1 do
    local particle = self.all[i]
    particle.timer = particle.timer
    particle.timer = particle.timer - dt
    if particle.timer < 0 then
      particle.kill = true
    end
    if particle.kill == true then
      table.remove(self.all, i)
    end
    
    particle.x = particle.x + particle.vx * dt
    particle.y = particle.y + particle.vy * dt
  end
  
  -- not updating everyparticles
  
  local marked = {}
  for i=#self.all,1,-1 do
    marked[i] = true
    if math.random() * #self.all < 1000 then
      local particle = self.all[i]
      for i=1,#planets.all do
        local planet = planets.all[i]
        local collision = physics:gravity(particle,planet,dt)
        if collision then 
          particle.kill = true
          break
        end
      end
      for i=1,#blackholes.all do
        local blackhole = blackholes.all[i]
        local collision = physics:gravity(particle,blackhole,dt)
        if collision then 
          particle.kill = true
          break
        end
      end
      for i=1,#stars.all do
        local star = stars.all[i]
        local collision = physics:gravity(particle,star,dt)
        if collision then 
          particle.kill = true
          break
        end
      end
    end
  end
end

function particles:draw(colorPalettes,debug)
  for i=1,#self.all do
    local particle = self.all[i]
    local a = particle.timer / particle.duration * 255
    if debug == true then
      love.graphics.setColor(255,255,255,a)
    else
      love.graphics.setColor(particle.c[1], particle.c[2], particle.c[3], a)
    end
    love.graphics.points(particle.x,particle.y)
  end
end

return particles