local function update(self,dt)
  for i=1,#self.points do
    local point = self.points[i]
    point.phi = point.phi + self.precession * dt
    if point.phi > 360 then
      point.phi = point.phi - 360
    elseif point.phi < 0 then
      point.phi = point.phi + 360
    end
  end
end

local function draw(self, colorPalettes, debug)
  love.graphics.setColor(0,0,0)
  love.graphics.circle("fill",self.x,self.y,self.r)
  if debug == true then
    love.graphics.setColor(colorPalettes[self.palettes[1]][1])
    love.graphics.circle("line",self.x,self.y,self.r)
  else
    for i=1,#self.points do
      local point = self.points[i]
      --print(point.phi)
      if point.phi < 180 then
        local phi = math.rad(point.phi)
        local theta = math.rad(point.theta)
        local dx = math.sin(theta)*math.cos(phi)*self.r
        local dy = math.cos(theta)*self.r
        
        --for k,v in pairs(self.palettes) do
        --  print(k..v)
        --end
        --print(point.color)
          
        love.graphics.setColor(colorPalettes[self.palettes[point.palette]][point.color])
        love.graphics.points(self.x + dx, self.y + dy)
      end
    end
  end
end

local function addColors(self,c)
  local index = #self.palettes
  for i=1,#c do
    self.palettes[index + i] = c[i]
  end
  
  for i=1,#self.points do
    local point = self.points[i]
    point.palette = math.random(1,#self.palettes)
    point.color = math.random(1,5)
  end
end


local planetFactory = {}

function planetFactory:new(x,y,r,vx,vy,c,p)
  local planet = {}
  
  planet.update = update
  planet.draw = draw
  planet.addColors = addColors
  
  planet.kill = false
  
  planet.x = x or 0
  planet.y = y or 0
  planet.vx = vx or 0
  planet.vy = vy or 0
  
  planet.palettes = {}
  
  planet.r = r or 100
  planet.m = planet.r
  planet.precession = p or math.random(-180,180)
  
  local nbPoints = math.floor(math.pow(planet.r*400,0.7))
  planet.points = {}
  for i=1, nbPoints do
    local point = {}
    point.phi = math.random(1,360) -- horizontal
    point.theta = math.random(1,180) -- vertical
    table.insert(planet.points,point)
  end
  
  local color = c or {"neutral"}
  planet:addColors(color)
  
  return planet
end

return planetFactory