local planetFactory = require("planetFactory")

local planets = {}

function planets:init()
  self.all = {}
end

function planets:new(x,y,r,vx,vy,c,p)
  local planet = planetFactory:new(x,y,r,vx,vy,c,p)
  table.insert(self.all, planet)
  return planet
end

function planets:explode(particles,colorPalettes,planet,power)  
  local ratio = 0
  for i=1,#planet.points do
    if math.random() > ratio then
      local point = planet.points[i]
      -- calculate
      local phi = math.rad(point.phi)
      local theta = math.rad(point.theta)
      local dx = math.sin(theta)*math.cos(phi)*planet.r
      local dy = math.cos(theta)*planet.r
      local x = planet.x + dx
      local y = planet.y + dy
      local v = math.random(1,power)
      local a = math.rad(math.random(1,360))
      local vx = planet.vx + v * math.cos(a)
      local vy = planet.vy + v * math.sin(a)
      local c = colorPalettes[planet.palettes[point.palette]][point.color]
      particles:new(x, y, vx, vy, c)
    end
  end
end

function planets:collision(particles,colorPalettes,a,b)
  local loss = 0.8
  
  local dx = b.x - a.x
  local dy = b.y - a.y
  local theta = math.atan2(dy,dx) -- collision angle
  local totalMass = a.m + b.m
  a.x = b.x - math.cos(theta) * (totalMass + 1)
  a.y = b.y - math.sin(theta) * (totalMass + 1)
  
  local va = math.sqrt(a.vx*a.vx + a.vy*a.vy) -- velocity module
  local phi1a = math.atan2(a.vy, a.vx) -- velocity angle
  local phi2a = phi1a - theta -- angle to the normal plan
  local vna = math.cos(phi2a) * va -- normal velocity 
  local vta = math.sin(phi2a) * va -- tangent velocity
  
  local vb = math.sqrt(b.vx*b.vx + b.vy*b.vy)
  local phi1b = math.atan2(b.vy, b.vx)
  local phi2b = phi1b - theta
  local vnb = math.cos(phi2b) * vb
  local vtb = math.sin(phi2b) * vb
  
  -- new normal comps
  local una = ((a.m - b.m)*vna + 2*b.m*vnb)/totalMass
  local unb = (a.m*(2*vna-vnb) + b.m*vnb)/totalMass
  
  -- new modules and args
  local ua = math.sqrt(una*una + vta*vta) * loss
  local phi3a = math.atan2(vta, una)
  
  local ub = math.sqrt(unb*unb + vtb*vtb) * loss
  local phi3b = math.atan2(vtb, unb)
  
  --print(unb.." "..vtb.." "..math.deg(phi3b + theta))
  --print(ua * math.cos(phi3a + theta).." "..ua * math.sin(phi3a + theta))
  --print(ub * math.cos(phi3b + theta).." "..ub * math.sin(phi3b + theta))
  
  -- new x,y composants
  
  
  
  
  -- shall one or both planets explose ?
  if a.m > b.m then
    b.kill = true
    a.vx = ua * math.cos(phi3a + theta)
    a.vy = ua * math.sin(phi3a + theta)
    a:addColors(b.palettes)
    self:explode(particles,colorPalettes,b,60)
  else
    a.kill = true
    b.vx = ub * math.cos(phi3b + theta)
    b.vy = ub * math.sin(phi3b + theta)
    b:addColors(a.palettes)
    self:explode(particles,colorPalettes,a,60)
  end
end

function planets:update(particles,physics,blackholes,stars,colorPalettes,dt)
  local marked = {}
  
  for i=#self.all,1,-1 do
    local planet = self.all[i]
    planet:update(dt) -- rotation only
    for i=1,#blackholes.all do
      local blackhole = blackholes.all[i]
      local collision = physics:gravity(planet, blackhole, dt)
      if collision then
        planets:explode(particles,colorPalettes,planet,0)
        planet.kill = true
      end
    end
    for i=1,#stars.all do
      local star = stars.all[i]
      local collision = physics:gravity(planet, star, dt)
      if collision then
        planets:explode(particles,colorPalettes,planet,0)
        planet.kill = true
      end
    end
    marked[i] = true

    for j=1,#self.all do
      if marked[j] ~= true then
        local other = self.all[j]
        local collision = physics:gravity(planet,other,dt,true)
        
        if collision then
          self:collision(particles,colorPalettes,planet,other)
        end
      end
    end
    
    planet.x = planet.x + planet.vx * dt
    planet.y = planet.y + planet.vy * dt    
    
    if planet.kill then
      table.remove(self.all, i)
    end
  end
end

function planets:draw(colorPalettes, camera, w, h, debug)  
  for i=1,#self.all do
    local planet = self.all[i]
    if planet.x + planet.r > camera.x 
    and planet.x - planet.r < camera.x + w / camera.scale
    and planet.y + planet.r > camera.y 
    and planet.y - planet.r < camera.y + h / camera.scale
    then 
      planet:draw(colorPalettes, debug)
    end
  end    
end

return planets