local spawner = {}

function spawner:init(what)
  self.x = 0
  self.y = 0
  self.t = 0
  self.velocity = 4
  self.radiusSpeed = 100
  self.spawning = false
  self.selected = what
end

function spawner:update(dt)
  if self.spawning then self.t = self.t + (self.t + dt) / 30 end
end

function spawner:pressed(camera,x,y)
  if self.spawning == false then
    self.x = x / camera.scale
    self.y = y / camera.scale
    self.t = 0
    self.spawning = true
  end
end

function spawner:change(what)
  self.selected = what
end

function spawner:released(camera,colors,planets,blackholes,stars,x,y)
  local dx = x / camera.scale - self.x
  local dy = y / camera.scale - self.y
  local vx = self.velocity * self.t * dx * camera.scale * 30
  local vy = self.velocity * self.t * dy * camera.scale * 30
  local r = self.t * self.radiusSpeed
  vx = vx / r
  vy = vy / r
  if camera.followed then
    vx = vx + camera.followed.vx
    vy = vy + camera.followed.vy
  end
  local xp = self.x + camera.x
  local yp = self.y + camera.y
  
  for i=1,#planets.all do
    local p = planets.all[i]
    local dx = p.x - xp
    local dy = p.y - yp
    local d = math.sqrt(dx*dx + dy*dy)
    if d < p.r then
      self.spawning = false
      return
    end
  end
  
  for i=1,#stars.all do
    local s = stars.all[i]
    local dx = s.x - xp
    local dy = s.y - yp
    local d = math.sqrt(dx*dx + dy*dy)
    if d < s.r then
      self.spawning = false
      return
    end
  end
  
  for i=1,#blackholes.all do
    local b = blackholes.all[i]
    local dx = b.x - xp
    local dy = b.y - yp
    local d = math.sqrt(dx*dx + dy*dy)
    if d < b.r then
      self.spawning = false
      return
    end
  end
  
  if self.selected == "planet" then
    -- random color palette
    local names = {}
    for k,v in pairs(colors) do
      table.insert(names,k)
    end
    local color = names[math.random(1,#names)]
    -- spawn
    planets:new(xp,yp,r,vx,vy,{color})
    
  elseif self.selected == "blackhole" then
    blackholes:new(xp,yp,r,vx,vy)
    
  elseif self.selected == "star" then
    stars:new(xp,yp,r,vx,vy)
  end
  
  self.spawning = false
end

function spawner:draw(camera)
  if self.spawning then
    local x = self.x + camera.x
    local y = self.y + camera.y
    love.graphics.setColor(255,255,255)
    love.graphics.circle("line",x,y,self.t*self.radiusSpeed)
    love.graphics.setColor(0,255,0,128)
    love.graphics.line(x,y,love.mouse.getX()/camera.scale+camera.x,love.mouse.getY()/camera.scale+camera.y)
  end
end

return spawner